﻿using Dynamicweb.Core;
using Dynamicweb.Core.UI.Icons;
using Dynamicweb.Ecommerce.FieldTypeProviders.ParameterEditors;
using Dynamicweb.Ecommerce.Products;
using Dynamicweb.Extensibility.AddIns;
using Dynamicweb.Extensibility.Editors;
using Dynamicweb.SystemTools;
using Dynamicweb.UI.Controls;
using System;
using System.Collections;
using System.Collections.Generic;
using System.Web.UI;

namespace Dynamicweb.Ecommerce.FieldTypeProviders
{
    [Serializable]
    public abstract class ConversionUnitFieldTypeProviderBase : ProductFieldTypeProvider, IDropDownOptions
    {
        public ConversionUnitFieldTypeProviderBase()
        {
            _options = new Lazy<Dictionary<string, ConversionUnit>>(InitalizeOptions);
        }

        [AddInParameter("Decimal places")]
        [AddInParameterEditor(typeof(IntegerNumberParameterEditor), "inputClass=inputControl")]
        [AddInParameterGroup("Source")]
        public int DecimalPlaces { get; set; }

        [AddInParameter("Language Units")]
        [AddInParameterEditor(typeof(LanguageDropDownList), "inputClass=inputControl")]
        [AddInParameterGroup("Source")]
        public string LanguageUnits { get; set; }

        private Lazy<Dictionary<string, ConversionUnit>> _options;

        internal abstract Dictionary<string, ConversionUnit> InitalizeOptions();

        public override string GetAppenderInfo(string languageId, string settings)
        {
            return GetAppenderInfo(languageId, GetParameterValues(settings));
        }

        public override string GetAppenderInfo(string languageId, Dictionary<string, string> settings)
        {
            string unitKey = GetUnitKey(languageId, settings);
            var unit = ConversionManager.GetUnitFromAbbreviation(unitKey, _options.Value);
            return unit != null ? unit.Symbol : unitKey;
        }
         
        public override string GetAppenderTitle(string languageId, Dictionary<string, string> settings)
        {
            string unitKey = GetUnitKey(languageId, settings);
            var unit = ConversionManager.GetUnitFromAbbreviation(unitKey, _options.Value);
            return unit != null ? unit.Name : unitKey;
        }

        private string GetUnitKey(string languageId, Dictionary<string, string> settings)
        {
            string unitLanguageKey = $"Unit_{languageId}";
            var parameterValues = GetLanguageUnits(settings);
            string unitKey = null;
            parameterValues.TryGetValue(unitLanguageKey, out unitKey);
            return unitKey;
        }

        private Dictionary<string, string> GetLanguageUnits(Dictionary<string, string> settings)
        {
            string languageUnitValue = string.Empty;
            if (settings.TryGetValue("Language Units", out languageUnitValue))
            {
                var unitValues = languageUnitValue.Split(new char[] { ',' }, StringSplitOptions.RemoveEmptyEntries);

                foreach (var unitValue in unitValues)
                {
                    var tokens = unitValue.Split('|');

                    if (tokens.Length == 2)
                    {
                        var valueKey = tokens[0];
                        string selectedUnit = tokens[1];
                        if(!settings.ContainsKey(valueKey))
                        {
                            settings.Add(valueKey, selectedUnit);
                        }
                    }
                }
            }

            return settings;
        }

        protected override Control Render(string id, bool enabled, string value, bool hasError, List<Dynamicweb.UI.Elements.Actions.FieldAddOn> addOns)
        {
            var input = new InputNumber();
            input.ID = id;
            input.Disabled = !enabled;
            input.Value = string.IsNullOrWhiteSpace(value) ? default(Decimal) : Converter.ToDecimal(value);
            input.IncrementSize = Converter.ToDecimal(0.01);

            foreach (var addonItem in addOns)
            {
                if (string.IsNullOrEmpty(addonItem.Text) && addonItem.Icon == KnownIcon.None)
                {
                    continue;
                }

                var addon = new FieldAddOn() { Id = addonItem.Id, Text = addonItem.Text, Icon = addonItem.Icon, IconColor = addonItem.IconColor, Title = addonItem.Title };
                if (addonItem.OnDefaultAction != null)
                {
                    addon.OnClick = addonItem.OnDefaultAction?.ToString();
                }
                input.AddOns.Add(addon);
            }

            if (hasError)
            {
                input.ValidationMessage = Translate.Translate("This field is required");
            }

            return input;
        }

        public override object GetValue(object value, string languageId, Dictionary<string, string> settings)
        {
            var parameterValues = GetLanguageUnits(settings);

            string fromUnit = null;
            string toUnit = null;
            string defaultLanguageId = Ecommerce.Services.Languages.GetDefaultLanguageId();
            string precisionValue = null;
            int precision = 0;
            if (parameterValues.TryGetValue("Decimal places", out precisionValue))
            {
                precision = Converter.ToInt32(precisionValue);
            }

            string fromUnitKey = $"Unit_{defaultLanguageId}";
            string toUnitKey = $"Unit_{languageId}";

            if (parameterValues.TryGetValue(toUnitKey, out toUnit) && parameterValues.TryGetValue(fromUnitKey, out fromUnit))
            {
                //There is no need to convert if to and from unit is the same
                if (String.Compare(toUnit, fromUnit, StringComparison.OrdinalIgnoreCase) != 0)
                {
                    var v = Converter.ToDecimal(value);
                    return Math.Round(ConversionManager.Convert(v, fromUnit, toUnit, _options.Value), precision);
                }
                else
                {
                    return Math.Round(Converter.ToDecimal(value), precision);
                }
            }

            return value;
        }

        public Hashtable GetOptions(string dropdownName)
        {
            // Get Hashtable
            Hashtable options = new Hashtable();

            foreach (var key in _options.Value.Keys)
            {
                options.Add(key, _options.Value[key].Name);
            }

            // Return the hashtable
            return options;
        }
    }
}
